@@ -1,4 +1,4 @@
-/* Copyright © 2007-2016 Evgeny Ratnikov
+/* Copyright © 2007-2023 Evgeny Ratnikov
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,13 +37,27 @@ struct TermitTab* termit_get_tab_by_index(guint index)
 {
     GtkWidget* tabWidget = gtk_notebook_get_nth_page(GTK_NOTEBOOK(termit.notebook), index);
     if (!tabWidget) {
-        ERROR("tabWidget at %u is NULL", index);
+        ERROR("tabWidget not found by index=%u", index);
         return NULL;
     }
     struct TermitTab* pTab = (struct TermitTab*)g_object_get_data(G_OBJECT(tabWidget), TERMIT_TAB_DATA);
     return pTab;
 }
 
+struct TermitTab* termit_get_tab_by_vte(VteTerminal* vte, gint* page)
+{
+    gint sz = gtk_notebook_get_n_pages(GTK_NOTEBOOK(termit.notebook));
+    for (gint p = 0; p < sz; ++p) {
+        struct TermitTab* pTab = termit_get_tab_by_index(p);
+        if (pTab && VTE_TERMINAL(pTab->vte) == vte) {
+            *page = p;
+            return pTab;
+        }
+    }
+    ERROR("tabWidget not found by vte=%p", vte);
+    return NULL;
+}
+
 static GtkWidget* create_statusbar()
 {
     GtkWidget* statusbar = gtk_statusbar_new();
@@ -147,14 +161,13 @@ static GtkWidget* termit_lua_menu_item_from_string(const gchar* label, const cha
 
 void termit_create_menubar()
 {
-
     GtkAccelGroup *accel = gtk_accel_group_new();
     gtk_window_add_accel_group(GTK_WINDOW(termit.main_window), accel);
     GtkWidget* menu_bar = gtk_menu_bar_new();
 
     // File menu
-    GtkWidget *mi_new_tab = termit_lua_menu_item_from_string(_("Open"), "openTab");
-    GtkWidget *mi_close_tab = termit_lua_menu_item_from_string(_("Delete"), "closeTab");
+    GtkWidget *mi_new_tab = termit_lua_menu_item_from_string(_("New tab"), "openTab");
+    GtkWidget *mi_close_tab = termit_lua_menu_item_from_string(_("Close tab"), "closeTab");
     GtkWidget *mi_exit = termit_lua_menu_item_from_string(_("Quit"), "quit");
 
     GtkWidget *mi_file = gtk_menu_item_new_with_label(_("File"));
@@ -199,7 +212,6 @@ void termit_create_menubar()
 
     // User menus
     termit_create_menus(menu_bar, accel, configs.user_menus);
-
     termit.menu_bar = menu_bar;
 }
 
@@ -207,8 +219,8 @@ void termit_create_popup_menu()
 {
     termit.menu = gtk_menu_new();
 
-    GtkWidget *mi_new_tab = termit_lua_menu_item_from_string(_("Open"), "openTab");
-    GtkWidget *mi_close_tab = termit_lua_menu_item_from_string(_("Delete"), "closeTab");
+    GtkWidget *mi_new_tab = termit_lua_menu_item_from_string(_("New tab"), "openTab");
+    GtkWidget *mi_close_tab = termit_lua_menu_item_from_string(_("Close tab"), "closeTab");
     GtkWidget *mi_set_tab_name = termit_lua_menu_item_from_string(_("Set tab name..."), "setTabTitleDlg");
     GtkWidget *mi_edit_preferences = termit_lua_menu_item_from_string(_("Preferences"), "preferencesDlg");
     GtkWidget *mi_copy = termit_lua_menu_item_from_string(_("Copy"), "copy");
@@ -263,6 +275,7 @@ static void termit_init(const gchar* initFile, gchar** argv)
     termit.tab_max_number = 1;
 
     termit.main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    gtk_window_set_default_icon_name("utilities-terminal");
     termit.statusbar = create_statusbar();
     create_search(&termit);
     termit.notebook = gtk_notebook_new();
@@ -460,7 +473,7 @@ int main(int argc, char **argv)
     g_signal_connect(G_OBJECT (termit.main_window), "destroy", G_CALLBACK (termit_on_destroy), NULL);
     g_signal_connect(G_OBJECT (termit.main_window), "key-press-event", G_CALLBACK(termit_on_key_press), NULL);
 
-    TRACE("window: came=[%s] class=[%s] role=[%s] title=[%s]",
+    TRACE("window: name=[%s] class=[%s] role=[%s] title=[%s]",
             windowName, windowClass, windowRole, windowTitle);
     if (windowName || windowClass) {
         gtk_window_set_wmclass(GTK_WINDOW(termit.main_window), windowName, windowClass);
@@ -476,6 +489,8 @@ int main(int argc, char **argv)
         gtk_window_set_title(GTK_WINDOW(termit.main_window), windowTitle);
         g_free(windowTitle);
     }
+    // Disable menubar on F10
+    g_object_set(G_OBJECT(gtk_widget_get_settings(termit.main_window)), "gtk-menu-bar-accel", "", NULL);
     /* Show the application window */
     gtk_widget_show_all(termit.main_window);
 
