@@ -1,4 +1,4 @@
-/* Copyright © 2007-2016 Evgeny Ratnikov
+/* Copyright © 2007-2023 Evgeny Ratnikov
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -120,11 +120,6 @@ static void termit_set_fonts()
     const gint width = (newWidth > oldWidth) ? newWidth : oldWidth;
     const gint height = (newHeight > oldHeight) ? newHeight : oldHeight;
     gtk_window_resize(GTK_WINDOW(termit.main_window), width, height);
-
-    GdkGeometry geom;
-    geom.min_width = width;
-    geom.min_height = height;
-    gtk_window_set_geometry_hints(GTK_WINDOW(termit.main_window), termit.main_window, &geom, GDK_HINT_MIN_SIZE);
 }
 
 gchar* termit_get_pid_dir(pid_t pid)
@@ -224,23 +219,37 @@ void termit_set_statusbar_message(guint page)
     }
 }
 
-static void termit_del_tab()
+void free_tab_data(struct TermitTab* pTab)
 {
-    gint page = gtk_notebook_get_current_page(GTK_NOTEBOOK(termit.notebook));
-
-    TERMIT_GET_TAB_BY_INDEX(pTab, page, return);
-    TRACE("%s pid=%d", __FUNCTION__, pTab->pid);
+    if (!pTab) {
+        return;
+    }
     g_array_free(pTab->matches, TRUE);
     g_free(pTab->encoding);
     g_strfreev(pTab->argv);
     g_free(pTab->title);
     termit_style_free(&pTab->style);
+    g_free(pTab->search_regex);
     g_free(pTab);
+}
+
+void termit_del_tab_n(gint page)
+{
+    TERMIT_GET_TAB_BY_INDEX(pTab, page, return);
+    TRACE("%s page=%d pid=%d", __FUNCTION__, page, pTab->pid);
+    g_signal_handler_disconnect(G_OBJECT(pTab->vte), pTab->onChildExitedHandlerId);
+    free_tab_data(pTab);
     gtk_notebook_remove_page(GTK_NOTEBOOK(termit.notebook), page);
 
     termit_check_tabbar();
 }
 
+static void termit_del_tab()
+{
+    gint page = gtk_notebook_get_current_page(GTK_NOTEBOOK(termit.notebook));
+    termit_del_tab_n(page);
+}
+
 static void termit_tab_add_matches(struct TermitTab* pTab, GArray* matches)
 {
     guint i = 0;
@@ -249,8 +258,8 @@ static void termit_tab_add_matches(struct TermitTab* pTab, GArray* matches)
         struct Match tabMatch = {};
         tabMatch.lua_callback = match->lua_callback;
         tabMatch.pattern = match->pattern;
-        tabMatch.tag = vte_terminal_match_add_gregex(VTE_TERMINAL(pTab->vte), match->regex, match->flags);
-        vte_terminal_match_set_cursor_type(VTE_TERMINAL(pTab->vte), tabMatch.tag, GDK_HAND2);
+        tabMatch.tag = vte_terminal_match_add_regex(VTE_TERMINAL(pTab->vte), match->regex, match->flags);
+        vte_terminal_match_set_cursor_name(VTE_TERMINAL(pTab->vte), tabMatch.tag, "pointer");
         g_array_append_val(pTab->matches, tabMatch);
     }
 }
@@ -275,7 +284,11 @@ static void termit_for_each_row_execute(struct TermitTab* pTab, glong row_start,
 {
     glong i = row_start;
     for (; i < row_end; ++i) {
+#if VTE_MAJOR_VERSION==0 && VTE_MINOR_VERSION<72
         char* str = vte_terminal_get_text_range(VTE_TERMINAL(pTab->vte), i, 0, i, 500, NULL, &lua_callback, NULL);
+#else
+        char* str = vte_terminal_get_text_range_format(VTE_TERMINAL(pTab->vte), VTE_FORMAT_TEXT, i, 0, i, 500, NULL);
+#endif
         str[strlen(str) - 1] = '\0';
         termit_lua_dofunction2(lua_callback, str);
         free(str);
@@ -321,24 +334,71 @@ void termit_tab_set_pos(struct TermitTab* pTab, int newPos)
     gtk_notebook_reorder_child(GTK_NOTEBOOK(termit.notebook), gtk_notebook_get_nth_page(GTK_NOTEBOOK(termit.notebook), index), newPos);
 }
 
+GtkWidget* termit_close_button(struct TermitTab* pTab)
+{
+    GtkWidget* button = (GtkWidget*)g_object_new(GTK_TYPE_BUTTON,
+            "relief", GTK_RELIEF_NONE, "focus-on-click", FALSE, NULL);
+    g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(termit_on_tab_close_clicked), pTab);
+    GIcon* icon = g_themed_icon_new_with_default_fallbacks("window-close-symbolic");
+    GtkWidget* image = gtk_image_new_from_gicon(icon, GTK_ICON_SIZE_MENU);
+    gtk_widget_show(image);
+    gtk_container_add(GTK_CONTAINER(button), image);
+    return button;
+}
+
+static void termit_on_spawn(VteTerminal* vte, GPid pid, GError* error, gpointer user_data)
+{
+    struct TermitTab* pTab = user_data;
+    if (pid == -1) {
+        return;
+    }
+    pTab->pid = pid;
+    TRACE("command=%s pid=%d", pTab->argv[0], pTab->pid);
+
+    g_signal_connect(G_OBJECT(pTab->vte), "bell", G_CALLBACK(termit_on_beep), pTab);
+    g_signal_connect(G_OBJECT(pTab->vte), "focus-in-event", G_CALLBACK(termit_on_focus), pTab);
+    g_signal_connect(G_OBJECT(pTab->vte), "window-title-changed", G_CALLBACK(termit_on_tab_title_changed), NULL);
+
+    pTab->onChildExitedHandlerId = g_signal_connect(G_OBJECT(pTab->vte), "child-exited", G_CALLBACK(termit_on_child_exited), NULL);
+    g_signal_connect_swapped(G_OBJECT(pTab->vte), "button-press-event", G_CALLBACK(termit_on_popup), NULL);
+
+}
+
 void termit_append_tab_with_details(const struct TabInfo* ti)
 {
     struct TermitTab* pTab = g_malloc0(sizeof(struct TermitTab));
     termit_style_copy(&pTab->style, &configs.style);
+    GtkWidget* label = NULL;
     if (ti->name) {
-        pTab->tab_name = gtk_label_new(ti->name);
+        label = gtk_label_new(ti->name);
         pTab->custom_tab_name = TRUE;
     } else {
         gchar* label_text = g_strdup_printf("%s %d", configs.default_tab_name, termit.tab_max_number++);
-        pTab->tab_name = gtk_label_new(label_text);
+        label = gtk_label_new(label_text);
         g_free(label_text);
         pTab->custom_tab_name = FALSE;
     }
     if (configs.tab_pos == GTK_POS_RIGHT) {
-        gtk_label_set_angle(GTK_LABEL(pTab->tab_name), 270);
+        gtk_label_set_angle(GTK_LABEL(label), 270);
     } else if (configs.tab_pos == GTK_POS_LEFT) {
-        gtk_label_set_angle(GTK_LABEL(pTab->tab_name), 90);
+        gtk_label_set_angle(GTK_LABEL(label), 90);
     }
+    GtkWidget* box = (configs.tab_pos == GTK_POS_RIGHT || configs.tab_pos == GTK_POS_LEFT)
+        ? gtk_box_new(GTK_ORIENTATION_VERTICAL, 0)
+        : gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+    gtk_box_set_center_widget(GTK_BOX(box), label);
+
+    if (configs.tab_close_button) {
+        GtkWidget* close_button = termit_close_button(pTab);
+        if (configs.tab_pos == GTK_POS_LEFT) {
+            gtk_box_pack_start(GTK_BOX(box), close_button, FALSE, FALSE, 0);
+        } else {
+            gtk_box_pack_end(GTK_BOX(box), close_button, FALSE, FALSE, 0);
+        }
+    }
+    gtk_widget_show_all(box);
+    pTab->tab_name = box;
+
     pTab->encoding = (ti->encoding) ? g_strdup(ti->encoding) : g_strdup(configs.default_encoding);
     pTab->bksp_binding = ti->bksp_binding;
     pTab->delete_binding = ti->delete_binding;
@@ -350,6 +410,8 @@ void termit_append_tab_with_details(const struct TabInfo* ti)
 
     vte_terminal_set_size(vte, configs.cols, configs.rows);
     vte_terminal_set_scrollback_lines(vte, configs.scrollback_lines);
+    vte_terminal_set_scroll_on_output(vte, configs.scroll_on_output);
+    vte_terminal_set_scroll_on_keystroke(vte, configs.scroll_on_keystroke);
     if (configs.default_word_char_exceptions) {
         vte_terminal_set_word_char_exceptions(vte, configs.default_word_char_exceptions);
     }
@@ -359,6 +421,18 @@ void termit_append_tab_with_details(const struct TabInfo* ti)
     vte_terminal_set_cursor_blink_mode(vte, pTab->cursor_blink_mode);
     vte_terminal_set_cursor_shape(vte, pTab->cursor_shape);
     vte_terminal_search_set_wrap_around(vte, TRUE);
+    vte_terminal_set_allow_hyperlink(vte, TRUE); // FIXME
+
+    GError *cmd_err = NULL;
+    if (vte_terminal_set_encoding(vte, pTab->encoding, &cmd_err) != TRUE) {
+        ERROR("cannot set encoding (%s): %s", pTab->encoding, cmd_err->message);
+        g_error_free(cmd_err);
+        return;
+    }
+
+    pTab->matches = g_array_new(FALSE, TRUE, sizeof(struct Match));
+    termit_tab_add_matches(pTab, configs.matches);
+    vte_terminal_set_font(vte, pTab->style.font);
 
     guint l = 0;
     if (ti->argv == NULL) {
@@ -377,7 +451,6 @@ void termit_append_tab_with_details(const struct TabInfo* ti)
     }
     g_assert(l >= 1);
     /* parse command */
-    GError *cmd_err = NULL;
     if (l == 1) { // arguments may be in one compound string
         gchar **cmd_argv;
         if (!g_shell_parse_argv(pTab->argv[0], NULL, &cmd_argv, &cmd_err)) {
@@ -391,43 +464,21 @@ void termit_append_tab_with_details(const struct TabInfo* ti)
     gchar *cmd_path = NULL;
     cmd_path = g_find_program_in_path(pTab->argv[0]);
 
-    TRACE("command=%s cmd_path=%s", pTab->argv[0], cmd_path);
     if (cmd_path != NULL) {
         g_free(pTab->argv[0]);
         pTab->argv[0] = g_strdup(cmd_path);
         g_free(cmd_path);
     }
-    if (vte_terminal_spawn_sync(vte,
+    vte_terminal_spawn_async(vte,
             VTE_PTY_DEFAULT,
             ti->working_dir,
             pTab->argv, NULL,
-            0, NULL, NULL,
-            &pTab->pid,
+            0,
+            NULL, NULL, NULL,
+            -1, // default timeout
             NULL, // g_cancellable_new
-            &cmd_err) != TRUE) {
-        ERROR("failed to open tab: %s", cmd_err->message);
-        g_error_free(cmd_err);
-        return;
-    }
-
-    g_signal_connect(G_OBJECT(pTab->vte), "bell", G_CALLBACK(termit_on_beep), pTab);
-    g_signal_connect(G_OBJECT(pTab->vte), "focus-in-event", G_CALLBACK(termit_on_focus), pTab);
-    g_signal_connect(G_OBJECT(pTab->vte), "window-title-changed", G_CALLBACK(termit_on_tab_title_changed), NULL);
-
-    g_signal_connect(G_OBJECT(pTab->vte), "child-exited", G_CALLBACK(termit_on_child_exited), NULL);
-//    g_signal_connect(G_OBJECT(pTab->vte), "eof", G_CALLBACK(termit_eof), NULL);
-    g_signal_connect_swapped(G_OBJECT(pTab->vte), "button-press-event", G_CALLBACK(termit_on_popup), NULL);
-
-    if (vte_terminal_set_encoding(vte, pTab->encoding, &cmd_err) != TRUE) {
-        ERROR("cannot set encoding (%s): %s", pTab->encoding, cmd_err->message);
-        g_error_free(cmd_err);
-        return;
-    }
-
-    pTab->matches = g_array_new(FALSE, TRUE, sizeof(struct Match));
-    termit_tab_add_matches(pTab, configs.matches);
-    vte_terminal_set_font(vte, pTab->style.font);
-
+            termit_on_spawn,
+            pTab);
     gint index = gtk_notebook_append_page(GTK_NOTEBOOK(termit.notebook), pTab->hbox, pTab->tab_name);
     if (index == -1) {
         ERROR("%s", _("Cannot create a new tab"));
@@ -518,7 +569,8 @@ void termit_tab_set_title(struct TermitTab* pTab, const gchar* title)
         g_free(pTab->title);
     }
     pTab->title = tmp_title;
-    gtk_label_set_text(GTK_LABEL(pTab->tab_name), pTab->title);
+    GtkWidget* label = gtk_box_get_center_widget(GTK_BOX(pTab->tab_name));
+    gtk_label_set_text(GTK_LABEL(label), pTab->title);
     termit_set_window_title(title);
 }
 
@@ -591,7 +643,7 @@ void termit_copy()
 {
     gint page = gtk_notebook_get_current_page(GTK_NOTEBOOK(termit.notebook));
     TERMIT_GET_TAB_BY_INDEX(pTab, page, return);
-    vte_terminal_copy_clipboard(VTE_TERMINAL(pTab->vte));
+    vte_terminal_copy_clipboard_format(VTE_TERMINAL(pTab->vte), VTE_FORMAT_TEXT);
 }
 
 static void clipboard_received_text(GtkClipboard *clipboard, const gchar *text, gpointer data)
@@ -622,6 +674,7 @@ void termit_close_tab()
 {
     termit_del_tab();
     if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(termit.notebook)) == 0) {
+        TRACE("no pages left, exiting");
         termit_quit();
     }
 }
